/* eslint-disable max-len */
let postcss = require('postcss')

module.exports = postcss.plugin('postcss-annotations-parser', options => {
  options = options || { preserve: true }

  // is this a CSS var fallback annotation
  function isCssVarFallback (annotation) {
    return !!annotation.text.includes('@css-var-fallback')
  }

  return css => {
    css.walkRules(rule => {
      rule.walkDecls(decl => {
        // if the previous line is a CSS var fallback annotation, then do some work
        let annotation = decl.prev()

        if (annotation && annotation.type === 'comment' && isCssVarFallback(annotation)) {
          // get the fallback property from the annotation
          let regex = /@(?=\S*['-])([a-zA-Z'-]+)\s(?=\S*['-]*)([a-zA-Z'-]+)/
          let matches = annotation.text.match(regex)

          // prepend this declaration with a fallback declaration
          if (matches && typeof matches[2] !== 'undefined') {
            // default is to preserve existing declaration
            if (options.preserve) {
              decl.cloneBefore({
                prop: matches[2],
                value: decl.value
              })
            } else {
              // else we overwrite the existing declaration
              decl.prop = matches[2]
            }
          }

          // remove the annotation
          annotation.remove()
        }
      })
    })
  }
})
